package org.eclipse.emf.example.tree;


import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.eclipse.emf.common.ui.ImageURIRegistry;
import org.eclipse.emf.common.ui.viewer.ColumnViewerInformationControlToolTipSupport;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;


public class View extends ViewPart
{
  public static final String ID = "org.eclipse.emf.example.tree.view";

  private TreeViewer viewer;
  private ColumnViewerInformationControlToolTipSupport.PathLocationListener locationListener;

  /**
   * The content provider class is responsible for providing objects to the
   * view. It can wrap existing objects in adapters or simply return objects
   * as-is. These objects may be sensitive to the current input of the view,
   * or ignore it and always show the same content (like Task List, for
   * example).
   */
  class ViewContentProvider implements ITreeContentProvider
  {
    public void inputChanged(Viewer v, Object oldInput, Object newInput)
    {
      // Ignore.
    }

    public void dispose()
    {
      // Ignore.
    }

    public Object[] getElements(Object parent)
    {
      return ((Folder)parent).contents.toArray();
    }

    public Object[] getChildren(Object parentElement)
    {
      return parentElement instanceof Folder ? ((Folder)parentElement).contents.toArray() : new Object[0];
    }

    public Object getParent(Object element)
    {
      return ((Resource)element).parent;
    }

    public boolean hasChildren(Object element)
    {
      return getChildren(element).length > 0;
    }
  }

  class ViewLabelProvider extends CellLabelProvider implements ILabelProvider
  {
    public String getText(Object element)
    {
      return ((Resource)element).name;
    }

    public Image getImage(Object element)
    {
      return PlatformUI.getWorkbench().getSharedImages().getImage(element instanceof Folder ? ISharedImages.IMG_OBJ_FOLDER : ISharedImages.IMG_OBJ_FILE);
    }

    @Override
    public String getToolTipText(Object element)
    {
      if (element instanceof File)
      {
        File file = (File)element;
        List<File> links = file.links;
        if (!links.isEmpty())
        {
          StringBuilder result = new StringBuilder("<b><a href='http://www.eclipse.org/emf'>Links</a></b>");
          for (File link : links)
          {
            // The SWT render used to compute the overall width removes image tags, so this adds enough white space to compensate, without changing the look of the actual HTML render.
            //
            result.append("<div style='position:absolute; left:0'>&#160;&#160;&#160;&#160;</div>");
            result.append("<div>");
            result.append("<img src='");
            result.append(ImageURIRegistry.INSTANCE.getImageURI(getImage(link)));
            result.append("'/>&nbsp;");
            result.append("<a href='");
            result.append(locationListener.getURI(link));
            result.append("'>");
            result.append(link.parent.name);
            result.append("/");
            result.append(link.name);
            result.append("</a>");
            result.append("</div>");
          }
          return result.toString();
        }
      }
      return null;
    }

    @Override
    public void update(ViewerCell cell)
    {
      Object element = cell.getElement();
      cell.setText(getText(element));
      cell.setImage(getImage(element));
    }
  }

  /**
   * This is a callback that will allow us to create the viewer and initialize
   * it.
   */
  @Override
  public void createPartControl(Composite parent)
  {
    viewer = new TreeViewer(parent);
    viewer.setContentProvider(new ViewContentProvider());
    viewer.setLabelProvider(new ViewLabelProvider());

    Folder root = new Folder();
    root.name = "root";
    List<File> files = new ArrayList<File>();
    for (int i = 0; i < 10; ++i)
    {
      Folder folder = new Folder();
      folder.name = "folder-" + i;
      folder.parent = root;
      root.contents.add(folder);

      for (int j = 0; j < 10; ++j)
      {
        File file = new File();
        file.name = "file-" + j;
        file.parent = folder;
        folder.contents.add(file);
        files.add(file);
      }
    }

    for (int i = 0, size = files.size(); i < size; ++i)
    {
      File file = files.get(i);
      Random random = new Random();
      for (int j = 0; j < 5; ++j)
      {
        file.links.add(files.get(Math.abs(random.nextInt()) % size));
      }
    }

    locationListener = new ColumnViewerInformationControlToolTipSupport.PathLocationListener(viewer);
    new ColumnViewerInformationControlToolTipSupport(viewer, locationListener);

    viewer.setInput(root);
  }

  /**
   * Passing the focus request to the viewer's control.
   */
  @Override
  public void setFocus()
  {
    viewer.getControl().setFocus();
  }
}

class Resource
{
  String name;
  Folder parent;
}

class Folder extends Resource
{
  List<Resource> contents = new ArrayList<Resource>();
}

class File extends Resource
{
  List<File> links = new ArrayList<File>();
}